#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `oceanum` package."""
import os
import pytest

from click.testing import CliRunner

from oceanum.datamesh import Connector, Datasource
from oceanum import cli


@pytest.fixture
def conn():
    """Connection fixture"""
    return Connector(os.environ["DATAMESH_TOKEN"])


def test_catalog(conn):
    cat = conn.get_catalog()
    ds0 = cat.ids[0]
    assert ds0 in str(cat)
    assert isinstance(cat[ds0], Datasource)
    assert len(cat)


@pytest.mark.asyncio
async def test_catalog_async(conn):
    cat = await conn.get_catalog_async()
    ds0 = cat.ids[0]
    assert ds0 in str(cat)
    assert isinstance(cat[ds0], Datasource)
    assert len(cat)


def test_command_line_interface():
    """Test the CLI."""
    runner = CliRunner()
    result = runner.invoke(cli.main)
    assert result.exit_code == 0
    assert "Oceanum.io CLI" in result.output
    help_result = runner.invoke(cli.main, ["--help"])
    assert help_result.exit_code == 0
    assert "--help  Show this message and exit." in help_result.output



import requests
import re
import time
import xarray
import numpy
from collections.abc import MutableMapping

from .exceptions import DatameshWriteError

try:
    import xarray_video as xv

    _VIDEO_SUPPORT = True
except:
    _VIDEO_SUPPORT = False


class ZarrClient(MutableMapping):
    def __init__(self, connection, datasource, method="post", retries=5):
        self.datasource = datasource
        self.method = method
        self.headers = {**connection._auth_headers, "cache-control": "no-transform"}
        self.gateway = connection._gateway + "/zarr"
        self.retries = retries

    def _get(self, path):
        retries = 0
        while retries < self.retries:
            try:
                resp = requests.get(path, headers=self.headers)
            except requests.RequestException:
                time.sleep(0.1 * 2**retries)
                retries += 1
            else:
                return resp

    def __getitem__(self, item):
        resp = self._get(f"{self.gateway}/{self.datasource}/{item}")
        if resp.status_code >= 300:
            raise KeyError(item)
        return resp.content

    def __setitem__(self, item, value):
        if self.method == "put":
            requests.put(
                f"{self.gateway}/{self.datasource}/{item}",
                data=value,
                headers=self.headers,
            )
        else:
            requests.post(
                f"{self.gateway}/{self.datasource}/{item}",
                data=value,
                headers=self.headers,
            )

    def __delitem__(self, item):
        requests.delete(
            f"{self.gateway}/{self.datasource}/{item}", headers=self.headers
        )

    def __iter__(self):
        resp = self._get(f"{self.gateway}/{self.datasource}")
        if not resp:
            return
        ex = re.compile(r"""<(a|A)\s+(?:[^>]*?\s+)?(href|HREF)=["'](?P<url>[^"']+)""")
        links = [u[2] for u in ex.findall(resp.text)]
        for link in links:
            yield link

    def __len__(self):
        return 0


def _to_zarr(data, store, **kwargs):
    if _VIDEO_SUPPORT:
        data.video.to_zarr(store, **kwargs)
    else:
        data.to_zarr(store, **kwargs)


def zarr_write(connection, datasource_id, data, append=None, overwrite=False):
    if overwrite is True:
        append = None
    else:
        ds = connection.get_datasource(datasource_id)
    store = ZarrClient(connection, datasource_id)
    if append and ds._exists:
        if append not in ds.dataschema.coords:
            raise DatameshWriteError(f"Append coordinate {append} not in existing zarr")
        with xarray.open_zarr(store) as dexist:
            cexist = dexist[append]
            if len(cexist.dims) > 1:
                raise DatameshWriteError(
                    f"Append coordinate {append} has more than one dimension"
                )
            append_dim = cexist.dims[0]
            (replace_range,) = numpy.nonzero(
                ((cexist >= data[append][0]) & (cexist <= data[append][-1])).values
            )  # Get range in new data which overlaps - this just replaces everything >= first value in the new data
            if len(replace_range):
                # Fail if the replacement range is larger than incomign data
                if len(replace_range) > len(data[append]):
                    raise DatameshWriteError(
                        f"Cannot append to zarr with a region that would be smaller than the original"
                    )

                drop_coords = [c for c in data.coords if c != append]
                replace_section = data.isel(
                    **{append_dim: slice(0, len(replace_range))}
                ).drop(drop_coords)
                replace_slice = slice(replace_range[0], replace_range[-1] + 1)
                # Fail if we are replacing an internal section and ends of coordinates do not match
                if replace_range[-1] + 1 < len(cexist) and not numpy.array_equal(
                    replace_section[append], cexist[replace_slice]
                ):
                    raise DatameshWriteError(
                        f"Data inconsistency on coordinate {append} replacing a inner section of an existing zarr array"
                    )
                _to_zarr(
                    replace_section,
                    store,
                    mode="a",
                    region={append_dim: replace_slice},
                )
            if len(data[append]) > len(replace_range):
                append_chunk = data.isel(
                    **{append_dim: slice(len(replace_range), None)}
                )
                _to_zarr(
                    append_chunk,
                    store,
                    mode="a",
                    append_dim=append_dim,
                    consolidated=True,
                )
    else:
        _to_zarr(data, store, mode="w", consolidated=True)
        ds = connection.get_datasource(datasource_id)
        ds.dataschema = data.to_dict(data=False)
    return ds



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `oceanum` package."""
import os
import pytest
import pandas
import geopandas
import xarray
import numpy

from click.testing import CliRunner

from oceanum.datamesh import Connector, Datasource
from oceanum.datamesh.exceptions import DatameshWriteError
from oceanum import cli

HERE = os.path.dirname(__file__)


@pytest.fixture
def conn():
    """Connection fixture"""
    return Connector()


@pytest.fixture
def dataframe():
    df = pandas.read_csv(
        os.path.join(HERE, "data", "point_data_1.csv"),
        parse_dates=True,
        index_col=0,
    )
    return df


@pytest.fixture
def dataset():
    ds = xarray.open_dataset(os.path.join(HERE, "data", "grid_data_1.nc"))
    return ds


def test_write_dataframe(conn, dataframe):
    datasource_id = "test-write-dataframe"
    conn.write_datasource(
        datasource_id, dataframe, {"type": "Point", "coordinates": [174, -39]}
    )
    df = conn.load_datasource(datasource_id)
    assert (df == dataframe).all().all()
    conn.delete_datasource(datasource_id)


def test_write_dataset(conn, dataset):
    datasource_id = "test-write-dataset"
    conn.write_datasource(datasource_id, dataset, overwrite=True)
    ds = conn.load_datasource(datasource_id)
    assert (ds == dataset).all()["u10"]
    conn.delete_datasource(datasource_id)


def test_append_dataset(conn, dataset):
    datasource_id = "test-write-dataset"
    dataset2 = dataset.copy()
    dataset2["time"] = dataset["time"] + numpy.timedelta64(1, "D")
    conn.write_datasource(datasource_id, dataset, overwrite=True)
    conn.write_datasource(datasource_id, dataset2, append="time")
    ds = conn.load_datasource(datasource_id)
    assert len(ds["u10"]) == 73
    assert (ds["u10"][:10] == dataset["u10"][:10]).all()
    conn.delete_datasource(datasource_id)


def test_append_dataset_fail(conn, dataset):
    datasource_id = "test-write-dataset"
    dataset2 = dataset.copy()
    dataset2["time"] = pandas.date_range(
        dataset["time"][10].values, dataset["time"][20].values, 49
    ).values
    conn.write_datasource(datasource_id, dataset, overwrite=True)
    with pytest.raises(DatameshWriteError):
        conn.write_datasource(datasource_id, dataset2, append="time")
    conn.delete_datasource(datasource_id)


def test_write_metadata(conn, dataframe):
    datasource_id = "test-write-dataframe"
    conn.write_datasource(
        datasource_id,
        None,
        name=datasource_id,
        coordinates={},
        driver="null",
        geometry={"type": "Point", "coordinates": [174, -39]},
        schema={"attrs": {}, "dims": {}, "coords": {}, "data_vars": {}},
        tstart="2020-01-01T00:00:00Z",
    )
    ds = conn.get_datasource(datasource_id)
    assert ds.name == datasource_id
    conn.delete_datasource(datasource_id)


def test_update_metadata(conn, dataframe):
    datasource_id = "test-write-dataframe2"
    conn.write_datasource(
        datasource_id, dataframe, {"type": "Point", "coordinates": [174, -39]}
    )
    conn.write_datasource(
        datasource_id,
        None,
        name="new name",
    )
    ds = conn.get_datasource(datasource_id)
    assert ds.name == "new name"
    conn.delete_datasource(datasource_id)
